/** sudp_send3 is sudp data sender server.
 *
 * Main diffrerence from 'sudp_send2', which is stable version for vdif sender 
 * used for Raipd-UT1 observation on Wz-Ts baseline, is using shared memory for 
 * monitoring of status by another program. Status of TransCtrl object can be 
 * monitored via shared memory.
 */

#include <iostream>
#include <vector>
#include <string>

#include <cstdio>
#include <cstring>
#include <cstdlib>
//#include <boost/algorithm/string.hpp>
using namespace std;

#include "SUDPProto.hpp"
//#include "strngutl.hpp"
#include "VLBI_Frame.hpp"
#include "VDIF_Frame.hpp"
#include "InFile_mdl.hpp"
#include "InMem_mdl.hpp"
#include "Trans_ctl.hpp"
//#include "k5vsi_mdl.hpp"
#include "StaticVals.hpp"
//#include "shem_lib.hpp"

extern "C"{
#include <signal.h>
}
char *me;  // My name

void usage(){
  cerr <<  "Usage: " << me 
       << " [-f <filename> | -m <sec> | -d <VSI|VSSP>]  [-p <server port>]\n";
  cerr << " [-R <Hz>] [-t <VDIF | RAW | K5VSSP>]  "  << endl; 
  cerr << " [-w <sec>] [-v <level>] [-i <input mode>] [-x <Normal|Mk5Geo>]\n";
  cerr << "-----------------------------\n";
  cerr << "  ==DATA SOURCE ==\n";
  cerr << "  -f: sending K5VSSP file with SUDP packets.\n";
    //       << " (if not specified, dummy data will be sent from memory.)\n";
  cerr << "  -m: sending dummy data for duration <sec>.\n";
  cerr << "  -d: Input device type.\n";
  //  cerr << "  -T: Data Frame Type for Input  (default: K5/VSSP32F) \n" ;
  cerr << "  ==Data Output==\n";
  cerr << "  -t: Data Frame Type For Output (default: VDIF) \n" ;
  cerr << "  -w: Page Reading Wait time (default: -1 endless) \n" ;
  //  cerr << "  -r: Rate Mbps (default: 100) \n" ;
  cerr << "  ==Control==\n";
  cerr << "  -p: Control Port of this host (default " << DEFAULT_SUDP_PORT
       << ") \n";
  cerr << "  -R: Rate Adjust Frequency (default 100 Hz)\n";
  cerr << "  -d: log directory path (default /tmp )\n";
  cerr << " -x: Extraction mode Normal(default) or Mk5Geo(stream 16-19/32 are LSB(14,15ch).\n";

  cerr << "  -i: Input mode in string, ex. 16Msps/2bit/4ch. \n";
  cerr << "    Default(VSSP): using header info.\n";
  cerr << "    Default(K5VSI): 32Msps/2bit/16ch.\n";
  cerr << "  -v: verbose level, 0--4 (default -1: no logging).\n";
  cerr << "  ----------- $Id: sudp_send2.cpp,v 1.6 2010/02/07 06:43:06 sekido Exp $ -------" << endl;
  exit(1); 
}

void LogSetup(const int verbose){

  string logdir = "/tmp/";
  string error_file = logdir + me +"_error.txt";
   
  string debug_file = logdir + me + "_dbg.txt";
  Logsystem::debug_logger()->set_stream(debug_file);
  string info_file = logdir + me + "_info.txt";
  Logsystem::info_logger()->set_stream(info_file);
  string warn_file = logdir + me + "_warn.txt";
  Logsystem::warning_logger()->set_stream(warn_file);
  string err_file = logdir + me + "_err.txt";
  Logsystem::error_logger()->set_stream(err_file);
      
  switch(verbose){
  case 0:
    Logsystem::debug_logger()->enable_logging(Logger::MINIMUM); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->set_stream(error_file);
    Logsystem::error_logger()->enable_logging();
    break;
  case 1:
    Logsystem::debug_logger()->enable_logging(Logger::SMALL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 2:
    Logsystem::debug_logger()->enable_logging(Logger::MEDIUM); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 3:
    Logsystem::debug_logger()->enable_logging(Logger::DETAIL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 4:
    Logsystem::debug_logger()->enable_logging(Logger::ALL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  default:
    cerr << " No Logging.\n";
    break;
  }
  
}


int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  string fname;
  string hostname("localhost");
  int srv_port(DEFAULT_CTRL_PORT);
  bool mflag(false); ///< Flag of memory data
  int mtu(DEFAULT_VDIF_FRAMESIZE +8);  
  int Mbps(100); // Mbps 
  int length(10); // sec
  int Rate_adj_f = 100; // (Hz)
  VLBI_Frame* out_vlbi_frame(0);
  VLBI_Frame* in_vlbi_frame(0);
  //  DAS_Info das_inf;
  int input_wait=-1;
  int verbose_level = -1;
  string device_type;
  DAS_Info indas( 32000000, 16, 2 );
  string inMode;
  bool   mk4geo_mode (false);
  
  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hf:d:p:m:t:w:v:R:i:x:")) { 
    case 'h':   usage();     break;
    case 'f':   fname = optarg; break;
    case 'd':   device_type = optarg; break;
      //    case 's':   mtu = atoi(optarg); break;
    case 't':   
      if(strncpy(optarg,"VDIF",4)==0){out_vlbi_frame = new VDIF_Frame; } 
      if(strncpy(optarg,"VSSP",4)==0){out_vlbi_frame = new K5VSSP32_Frame; } 
      break;
    case 'w':   input_wait = atoi(optarg); break;
    case 'i':   inMode = optarg; break;
      //    case 'r':   Mbps = atoi(optarg); break;
    case 'R':   Rate_adj_f = atoi(optarg); break;
    case 'm':   length  = atoi(optarg); mflag = true; break;
      //    case 'a':   hostname = optarg; break;
    case 'p':   srv_port  = atoi(optarg); break;
    case 'x':   if(strncmp(optarg,"Mk5Geo",6) == 0) 
	mk4geo_mode=true; break;
    case 'v':   verbose_level = atoi(optarg); break;
      //    case 'l':   length  = atoi(optarg); break;
    case '?':    usage(); break;
    }
  }

  if(mflag == false && fname.empty() && device_type.empty()){
    cerr << "Error: Input Filename/device/memory is not specified.\n";
    usage();
  }

  if( out_vlbi_frame == 0 ) out_vlbi_frame = new VDIF_Frame;
  indas.SetMode(inMode);
//   if( ! inMode.empty() ){
//     int pos = inMode.find("Msps/"); 
//     int sr = atoi(inMode.substr(0,pos).c_str());
//     indas.Set_Srate(sr*1000000);
//     string work = inMode.substr(pos+5);
//     pos = work.find("bit/"); sr = atoi(work.substr(0,pos).c_str());
//     indas.Set_Qbit(sr);
//     work = inMode.substr(pos+4);
//     pos = work.find("ch"); sr = atoi(work.substr(0,pos).c_str());
//     indas.Set_Nch(sr);
//   }

  LogSetup(verbose_level);

  /** Simple UDP Protocl for transmission */
  SUDPProto  sudp(mtu);
  sudp.Set_OprMode(TransInfo::TX); // Transmission mode 
  sudp.Set_Adj_Freq(Rate_adj_f);
  sudp.Set_Speed(Mbps,true); // Setti100 Mbps
  sudp.Disp_TxRate(); 
  sudp.Set_VlbiFrame(out_vlbi_frame); // Set Output Frame 
  if(mk4geo_mode){
    sudp.SetExmode_Mk4Geo(); // Extracting LSB 8,9ch->15,16ch
    cerr << "I: Tx mode- Mk4VSI-Geodesy" << endl;
  }
  

  /** Transport Control object */ 
  //  TransCtl transctl_proto;
  Shem_adm shem("/tmp", 1, sizeof(TransCtl));
  TransCtl* ptransctl_proto = (TransCtl*)shem.GetAddr(Shem::INI);
  
  ptransctl_proto->Set_CtrlPort(srv_port);
  //  out_vlbi_frame->Set_DasInfo(&das_inf); 
  //  in_vlbi_frame->Set_DasInfo(&das_inf); 
  
  //  transctl_proto.Set_DasInfo(&das_inf); // Set DasInfo
  ptransctl_proto->Set_ApplProto(&sudp); // Set TransInfo
  
  fprintf(stderr,"Server- Port: %d  MTU= %d \n",  srv_port, mtu);
  fprintf(stderr,"Initial Rate: %d Mbps For %d sec, Rate Adjust by %d cycle\n",
	  Mbps, length, Rate_adj_f);
  /***************************************/

  InDev_Model *input_mdl(0);
  try{
    /** Data Input Device/File */
    if(! fname.empty()){   // sending from file.
      input_mdl = new InFile_Model;
      in_vlbi_frame = new K5VSSP32_Frame;
      in_vlbi_frame->Set_DasInfo( indas );
      input_mdl->Set_VlbiFrame(in_vlbi_frame);
      ((InFile_Model*)input_mdl)->Set_infile(fname.c_str());  
      fprintf(stderr,"Data source : File(%s)  \n", fname.c_str()); 
    }else if(!device_type.empty()){ // Data input from Device
      //    if( strncmp(device_type.c_str(),"VSI",3) == 0) 
      input_mdl = new K5_VSI;
      in_vlbi_frame = new K5VSI_Frame;
      in_vlbi_frame->Set_DasInfo( indas );
      input_mdl->Set_VlbiFrame(in_vlbi_frame);
      //if( strncmp(device_type.c_str(),"VSSP",4) == 0) input_mdl = new K5_VSSP;
      fprintf(stderr,"Data source : Device(%s)  \n", device_type.c_str()); 
      if( input_mdl->TestDev() < 0){
	throw "Failed to open K5/VSI device file";
      }
    }else{ // data from memory
      input_mdl = new InMem_Model(length);
      in_vlbi_frame = new K5VSI_Frame;
      input_mdl->Set_VlbiFrame(in_vlbi_frame);
      fprintf(stderr,"Data source: Memory   \n") ;
    }
    //  input_mdl->Set_PageWait(input_wait);

    ptransctl_proto->Set_InputDev(input_mdl); // Set Input device 
    if(! ptransctl_proto->SetupCheck()) exit(1);
    //  input_mdl->Ready();
    //  input_mdl->start(); // reading data Started 

    //  signal(SIGINT,SIG_IGN);

    ptransctl_proto->start(); // Transaction server starting
  
    //  char sRecvBuf[256];
    //   while(ptransctl_proto->Status() == Thread::RUNNING ){
    //     fprintf(stderr,"Terminate by \"QUIT\" \n");
    //     scanf("%s", sRecvBuf);
    //     StrUpper(sRecvBuf,strlen(sRecvBuf));
    //     if(strncmp(sRecvBuf,"QUIT",4)==0){
    //       break;
    //     }
    //   }
  
    cerr << "TransCtrl Thread ...";
    ptransctl_proto->join();
    cerr << "joined\n";
  }// End of try

  catch(const char* msg){
    cerr << "Error:" << msg << endl;
  }  
  sudp.Disp_RateOff();  

  /** Memory Release */
  if( input_mdl != 0 ) delete input_mdl;
  if( in_vlbi_frame != 0 ) delete in_vlbi_frame;
  if( out_vlbi_frame != 0 ) delete out_vlbi_frame;
  /****************************/

  if(Logsystem::Test_Logsys(Logsystem::DEBUG)) 
    Logsystem::debug_logger()->close_stream();
  if(Logsystem::Test_Logsys(Logsystem::INFO))   
    Logsystem::info_logger()->close_stream();
  
  ptransctl_proto->Clean();  
  shem.Release();
}


