
/** @file vdif_chk is test reading VDIF header data 
 *  @author M.Sekido
 *  @Date 2009.4.6
 */

#include <iostream>
#include <string>
#include <vector>
using namespace std;

#include <cstdio>
#include <cstdlib>

#include "VDIF_Header.hpp"
#include "VDIF_Frame.hpp"
#include "DAS_info.hpp"
#include "VsiData.hpp"
#include "hist_g.hpp"

extern "C"{
#include <unistd.h>
}

/**! "main_cmn.hpp": Common Declraration of static parameters. 
 * Should be included in main file only. */
#include "main_cmn.hpp"


void usage(){
  cerr << "Usage: " << me << " -f <filename> [-l <level>] [-d] [-n <int>]\n";
  cerr << "        [-b <bit encode>] \n";
  cerr << "-l : Display level 0-4\n";
  cerr << "-n : Number of frames to be displayed. (default:10)\n";
  cerr << "-d : Detailed output for every frame rather than every secnd.\n";
  cerr << "     (default: is print header by every second.) \n";
  //  cerr << "-e : Extended version Specification (NICT or none). Default is none\n";
  cerr << "-b  : Choice of bit encoding for histgram.\n";
  cerr << "      one of [ADS2000/ADS3000/SignMag/RevSign].(default=ADS2000)\n";
  cerr << "           ADS2000: 00,01,10,11 from bottom to top level.\n";
  cerr << "           ADS3000: 00,10,01,11 from bottom to top level.\n";
  cerr << "           SignMag: 01,00,10,11 from bottom to top level.\n";
  cerr << "           RevSgn : 10,00,01,11 from bottom to top level.\n";
  exit(1);
}

int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */

  string fname;
  //  if(argc < 2) usage();
  //  int dlevel=1;
  //  VDIF_Ext_t ext_t = No_ext; // No VDIF-extended version
  size_t nsec = 0; // Endless display
  size_t ndisp = 10;
  bool every_frame = false;
  VsiData::BitEnc_t code_type = VsiData::ADS2000_2bit;

  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hf:b:dn:")) { 
    case 'h':  usage(); break;
    case 'f':    fname = optarg;      break;
    case 'd':    every_frame = true;      break;
    case 'b':
      if( strncmp(optarg, "ADS3000",7)==0) code_type = VsiData::ADS3000_2bit;
      if( strncmp(optarg, "SignMag",7)==0) code_type = VsiData::SignMag_2bit;
      if( strncmp(optarg, "RevSign",7)==0) code_type = VsiData::RevSgnMag_2bit;
      break;
    case 'n':    nsec = ndisp = atoi(optarg);      break;
      // case 'e':    if(strcmp(optarg,"NICT")==0) ext_t = NICT_ext; break;
      //    case 'l':    dlevel = atoi(optarg);      break;
      //    case 'd': vdif_hdr.Set_dbg(atoi(optarg)); break;
    case '?' : /* Error:  character not in list */ 
      fprintf(stderr, /* Yes */ 
	      "%s WARNING:  Unknown option on command line \n", me); 
      usage();
      break; 
    case ':' : /* Error:  missing parameter */ 
      fprintf(stderr, "%s WARNING:  Missing number on command line \n", me); 
      usage();
      break; 
    }
  }

  if(fname.empty()) usage();
  
  //  FILE *fp = fopen(fname.c_str(),"r");
  VDIF_Frame  vdif;
  if( ! vdif.ROpen(fname)){
    cerr << "Error in opening file: " << fname << endl;
    exit(1);
  }
  
  vector <VDIF_Hdr> hdrarr;
  //  uint32_t nframe = vdif.Read_every_sec_hdr(hdrarr, nsec);
  uint32_t nframe = vdif.Read_every_sec_hdr(hdrarr, 0, VDIF_Frame::QUICK);
  fprintf(stdout,"Total data length: %lu sec \n", hdrarr.size()); 

  if(nframe < 1){
    cerr << "E Failed to find VDIF Header.\n";
    exit(1);
  }
  
  long datsize = vdif.Framesize() - vdif.Headsize();
  int nch = hdrarr[0].Nch();
  int qbit = hdrarr[0].Qbit();
  int npt_frame = datsize * 8/qbit/nch; // #point per frame;
  int lbound(0), ndiv(2), istep(0);
  switch(qbit){
  case 1: lbound = 0, istep=2; ndiv=2; break;
  case 2: lbound =-2; istep=2; ndiv=4;  break;
  case 4: lbound =-7, istep=1; ndiv=16; break;
  case 8: lbound =-127, istep=1; ndiv=256; break;
  }
  Hist_g <int8_t> hist( lbound, ndiv,istep);
  
  cout << "========= VDIF check ver.3 ================\n";
  double srate = (double)nframe * datsize * 8/hdrarr.size()/qbit/nch*1.e-6;
  fprintf(stdout," Reading Nsec =%ld sec, %lu frames, Rate(Estimate) = %.3e Msps\n", 
	  hdrarr.size(), (unsigned long)nframe, srate); 
  string Info1 =  hdrarr[0].Str_hd( 20);
  fprintf(stdout, "%s \n",Info1.c_str()); 
  if( every_frame){
    vdif.Rewind();
    vector <uint8_t> rbuf;
    DAS_Info das = vdif.DAS_Inf();// Sampling info
    VsiData vsidat(&das);
    vsidat.Make_Table(code_type); // Make Decode Table 
    for(int i =0; i < (int)ndisp; i++){
      if(vdif.Read_frame(hdrarr, rbuf, 1) < 1){
	fprintf(stdout, "E Failed to read frame\n"); break;
      }
      string Info1_dmy = hdrarr[0].Str_hd(20);
      if(Info1 != Info1_dmy) fprintf(stdout, "%s \n",Info1_dmy.c_str()); 

      // Histgram Compute 
      vsidat.Interpret(rbuf,npt_frame);
      fprintf(stdout, "%s ", hdrarr[0].Str_hd(21).c_str());
      if(nch > 7)fputc('\n',stdout);
      fprintf(stdout,"Hist[ %% ]");
      for(int ich =0; ich<nch; ich++){
	hist.Init();
	hist.Accm(vsidat.ValArr(ich));
	string str = hist.StrRatioArr(); 
	fprintf(stdout,"(%dch):%s ", ich, str.c_str());
      }
      fputc('\n', stdout);
    }
  }else{
    for(int i=0; i < (int)hdrarr.size(); i++){
	hdrarr[i].Print_hd(stdout, 22);
	//fprintf(stdout," diff = %d\n", hdrarr[i].iFrame() - hdrarr[i-1].iFrame());
	//      if(ndisp !=0 && i > (int)ndisp) break;
    }
    
  }
  cout << "==========================================\n";

}


